@@ -1,7 +1,7 @@
 /**************************************************************************
  *   help.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 2000-2011, 2013-2024 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2000-2011, 2013-2025 Free Software Foundation, Inc.    *
  *   Copyright (C) 2017 Rishabh Dave                                      *
  *   Copyright (C) 2014-2019 Benno Schulenberg                            *
  *                                                                        *
@@ -16,7 +16,7 @@
  *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
+ *   along with this program.  If not, see https://gnu.org/licenses/.     *
  *                                                                        *
  **************************************************************************/
 
@@ -476,18 +476,11 @@ void show_help(void)
 
 #ifndef NANO_TINY
 		spotlighted = FALSE;
-
-		if (bracketed_paste || kbinput == BRACKETED_PASTE_MARKER) {
-			beep();
-			continue;
-		}
 #endif
 		function = interpret(kbinput);
 
-		if (function == full_refresh) {
-			full_refresh();
-		} else if (ISSET(SHOW_CURSOR) && (function == do_left || function == do_right ||
-											function == do_up || function == do_down)) {
+		if (ISSET(SHOW_CURSOR) && (function == do_left || function == do_right ||
+									function == do_up || function == do_down)) {
 			function();
 		} else if (function == do_up || function == do_scroll_up) {
 			do_scroll_up();
@@ -511,9 +504,15 @@ void show_help(void)
 			get_mouseinput(&dummy_row, &dummy_col, TRUE);
 #endif
 #ifndef NANO_TINY
-		} else if (kbinput == KEY_WINCH) {
+		} else if (kbinput == START_OF_PASTE) {
+			while (get_kbinput(midwin, BLIND) != END_OF_PASTE)
+				;
+			statusline(AHEM, _("Paste is ignored"));
+		} else if (kbinput == THE_WINDOW_RESIZED) {
 			;  /* Nothing to do. */
 #endif
+		} else if (function == full_refresh) {
+			full_refresh();
 		} else if (function == do_exit) {
 			break;
 		} else
